/* ex1_main
**
**
*/
new;
closeall;
cls;
library pgraph, user;

format /mat /on /mb1 /ros 10,3;
outwidth 256;

#include ex1_spec.g;
#include ex1_procs.g;
#include pathspec.g;


// Set random seed for simulation
rndseed 1234;

// Set some constants
simrun    = 2;
simrunstr = "simrun" $+ ftos(simrun,"%*.*lf",1,0);

if simrun == 1;
   nsim         = 200;
   nsim_thdens_ = 100;
   nblocks      = 10;
elseif simrun == 2;
   nsim         = 5000;
   nsim_thdens_ = 250;
   nblocks      = 100;
endif;

r_grid   = 0.01 | 0.2 | 0.3 | 0.4 | 0.5 | 0.6 | 0.7 | 0.8 | 0.9 | 0.999 |
           0.01 | 0.2 | 0.3 | 0.4 | 0.5 | 0.6 | 0.7 | 0.8 | 0.9 | 0.999 ;
           
tau_grid = zeros(10,1)   |
           0.5*ones(10,1); 
n_grid   = rows(r_grid);

lnpown = ln(0.52);
lnpref = ln(0.48);

// Intialize Matrices that contain the results
M1preferred    = zeros(nsim,n_grid);
Mtrue_vec      = zeros(nsim,1);
PMtheta        = zeros(nsim,n_grid);

M1reportstruth = zeros(nsim,n_grid);
M2reportstruth = zeros(nsim,n_grid);

PMfindsbest    = zeros(nsim,n_grid);
PMfindsbestth  = zeros(nsim,n_grid);
PMfindsbestmining = zeros(nsim,n_grid);

Selecterror      = zeros(nsim,n_grid);
Selecterrorth    = zeros(nsim,n_grid);

PMprecision    = zeros(nsim,n_grid);
PMprecisionth  = zeros(nsim,n_grid);
PMprecisionmining = zeros(nsim,n_grid);

PMfindsbest_alhon   = zeros(nsim,n_grid);
PMfindsbestth_alhon = zeros(nsim,n_grid);

PMtotalloss    = zeros(nsim,n_grid);
PMtotallossth  = zeros(nsim,n_grid);
PMtotallossmining = zeros(nsim,n_grid);

// start log file 
outstr = resupath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ simrunstr $+ ".log"; 
output file = ^outstr reset;
"Computing Integrated Risk";

// set start time 
tstart = date;

// Computing Integrated Risk
i = 1;
do until i > nsim;

   // Step 1: Generate Observations
   {YYC,YYT,XXC,XXT,SC,ST,theta0,beta0,Mtrue} = generatedata(pi01,siguu,lam1,lam2,n,sigmaxx,tsize);
   Mtrue_vec[i,.] = Mtrue;
   YY = YYC|YYT; S = SC|ST; 
   YY = YY - meanc(YY)'; S = S - meanc(S)';
   PMtheta[i,.] = ones(1,n_grid)* (inv(S'*S)*S'*YY);

   // Step 2: Full Bayesian Analysis
   {theta_bar1_bayes,V_bar1_bayes,theta_bar2_bayes,V_bar2_bayes,
    theta_bar10,theta_bar20,
    lnodds_bayes,delta_fit,delta_penalty} 
     = bayesiananalysis(YYC,YYT,XXC,XXT,SC,ST,pi01,lam1,lam2);

   M1preferred[i,.] = ones(1,n_grid)*(lnodds_bayes .ge 0);
   
   // fix regressors
   xbar1 = xbar_;  xbar2 = corrx1x2*xbar_;

   // Bayes predictor of policy effect
   if lnodds_bayes GE 0;
      //M1 is preferred
      //pred_bayes = (xbar_+sbar_)*theta_bar1_bayes;
      pred_bayes = (xbar1+sbar_)*theta_bar1_bayes;
      //xbar1 = xbar_; xbar2 = corrx1x2*xbar_;
   else;
      //M2 is preferred
      //pred_bayes = (xbar_+sbar_)*theta_bar2_bayes;
      pred_bayes = (xbar2+sbar_)*theta_bar2_bayes;
      //xbar2 = xbar_; xbar1 = corrx1x2*xbar_;
   endif;

   // Step 3: Full Mining
   {theta_bar1_mine,V_bar1_mine,theta_bar2_mine,V_bar2_mine,lnodds_mine,delta_fit_mine,delta_penalty_mine}
    = datamining(YYC,YYT,XXC,XXT,SC,ST,pi01);
   if lnodds_mine GE 0;
      //M1 is preferred
      pred_mine = (xbar1 ~ sbar_)*theta_bar1_mine;
   else;
      //M2 is preferred
      pred_mine = (xbar2 ~ sbar_)*theta_bar2_mine;
   endif;

   PMtotallossmining[i,.] = ones(1,n_grid)*n*(pred_mine-pred_bayes)^2;       

   if (lnodds_bayes > 0) and (lnodds_mine > 0);
       PMfindsbestmining[i,.] = ones(1,n_grid);
   elseif (lnodds_bayes < 0) and (lnodds_mine < 0);  
       PMfindsbestmining[i,.] = ones(1,n_grid);
   endif;

   j = 1;
   do until j > n_grid;

      r = r_grid[j];
      tau = tau_grid[j];

      // Step 3: Analysis Based on Split Sample 
      // tilde refers to actual model
      // bar refers to mining
      {lnpYY_tilde1_r,lnpYY_bar1_r,lnpYY_tilde2_r,lnpYY_bar2_r,
       theta_tilde1_r,theta_tilde2_r,theta_bar1_r,theta_bar2_r,
       mth_m1tilde,sth_m1tilde,mth_m1bar,sth_m1bar,
       mth_m2tilde,sth_m2tilde,mth_m2bar,sth_m2bar}
        = samplesplitting(YYC,YYT,XXC,XXT,SC,ST,lam1,lam2,r,tau);

      //if Mtrue == 1;
      //   r PMtheta[i,j] mth_m1tilde;
      //else;
      //   r PMtheta[i,j] mth_m2tilde;
      //endif;
      //pause(1);

      // Modelers examine odds versus reference model 
      //M1reportstruth[i,j] = (lnpYY_tilde1_r+lnpown) .ge (lnpYY_bar1_r+lnpref);
      //M2reportstruth[i,j] = (lnpYY_tilde2_r+lnpown) .ge (lnpYY_bar2_r+lnpref);

      // Modelers always use their assigned model
      M1reportstruth[i,j] = 1;
      M2reportstruth[i,j] = 1;
      
      lnodds_pred = lnodds_bayes 
                    - maxc(lnpYY_tilde1_r | lnpYY_bar1_r)
                    + maxc(lnpYY_tilde2_r | lnpYY_bar2_r);

      if M1reportstruth[i,j] == 1;
         thdens_m1 = -ln(sth_m1tilde) - 0.5*(PMtheta[i,j]-mth_m1tilde)^2/sth_m1tilde^2;
         pred_m1   = (xbar1 + sbar_)*theta_bar1_bayes; //theta_tilde1_r;
      else; 
         thdens_m1 = -ln(sth_m1bar) - 0.5*(PMtheta[i,j]-mth_m1bar)^2/sth_m1bar^2;
         pred_m1   = (xbar1 ~ sbar_)*theta_bar10; //theta_bar1_r;
      endif;

      if M2reportstruth[i,j] == 1;
         thdens_m2 = -ln(sth_m2tilde) - 0.5*(PMtheta[i,j]-mth_m2tilde)^2/sth_m2tilde^2;
         pred_m2   = (xbar2 + sbar_)*theta_bar2_bayes; //theta_tilde2_r;
      else; 
         thdens_m2 = -ln(sth_m2bar) - 0.5*(PMtheta[i,j]-mth_m2bar)^2/sth_m2bar^2;
         pred_m2   = (xbar2 ~ sbar_)*theta_bar20; //theta_bar2_r;
      endif;

      // Determin PM loss based on Yp and theta-hat selection
      
      if (lnodds_pred GE 0);
         PMtotalloss[i,j] = n*(pred_m1-pred_bayes)^2;       
      else;
         PMtotalloss[i,j] = n*(pred_m2-pred_bayes)^2;       
      endif;      

      if (thdens_m1 GE thdens_m2);       
         PMtotallossth[i,j] = n*(pred_m1-pred_bayes)^2;       
      else;
         PMtotallossth[i,j] = n*(pred_m2-pred_bayes)^2;       
      endif;
      
      // PM finds best

      if (lnodds_bayes > 0) and (lnodds_pred > 0) and (M1reportstruth[i,j]==1);
          PMfindsbest[i,j] = 1;
      elseif (lnodds_bayes < 0) and (lnodds_pred < 0) and (M2reportstruth[i,j]==1);  
          PMfindsbest[i,j] = 1;
      endif;

      if (lnodds_bayes > 0) and (thdens_m1 > thdens_m2) and (M1reportstruth[i,j]==1);
          PMfindsbestth[i,j] = 1;
      elseif (lnodds_bayes < 0) and (thdens_m1 < thdens_m2) and (M2reportstruth[i,j]==1);  
          PMfindsbestth[i,j] = 1;
      endif;
      
      // Seclection error
      if (lnodds_bayes > 0) and (lnodds_pred > 0) and (M1reportstruth[i,j]==0);
          Selecterror[i,j] = 1;
      elseif (lnodds_bayes < 0) and (lnodds_pred < 0) and (M2reportstruth[i,j]==0);  
          Selecterror[i,j] = 1;
      elseif (lnodds_bayes > 0) and (lnodds_pred < 0) and (M2reportstruth[i,j]==1);
          Selecterror[i,j] = 2;
      elseif (lnodds_bayes < 0) and (lnodds_pred > 0) and (M1reportstruth[i,j]==1);
          Selecterror[i,j] = 2;
      elseif (lnodds_bayes > 0) and (lnodds_pred < 0) and (M2reportstruth[i,j]==0);
          Selecterror[i,j] = 3;
      elseif (lnodds_bayes < 0) and (lnodds_pred > 0) and (M1reportstruth[i,j]==0);
          Selecterror[i,j] = 3;
      endif;

      if (lnodds_bayes > 0) and (thdens_m1 > thdens_m2) and (M1reportstruth[i,j]==0);
          Selecterrorth[i,j] = 1;
      elseif (lnodds_bayes < 0) and (thdens_m1 < thdens_m2) and (M2reportstruth[i,j]==0);  
          Selecterrorth[i,j] = 1;
      elseif (lnodds_bayes > 0) and (thdens_m1 < thdens_m2) and (M2reportstruth[i,j]==1);
          Selecterrorth[i,j] = 2;
      elseif (lnodds_bayes < 0) and (thdens_m1 > thdens_m2) and (M1reportstruth[i,j]==1);
          Selecterrorth[i,j] = 2;
      elseif (lnodds_bayes > 0) and (thdens_m1 < thdens_m2) and (M2reportstruth[i,j]==0);
          Selecterrorth[i,j] = 3;
      elseif (lnodds_bayes < 0) and (thdens_m1 > thdens_m2) and (M1reportstruth[i,j]==0);
          Selecterrorth[i,j] = 3;
      endif;



      // PM finds best, assuming modelers are honest

      if (lnodds_bayes > 0) and (lnodds_pred > 0);
          PMfindsbest_alhon[i,j] = 1;
      elseif (lnodds_bayes < 0) and (lnodds_pred < 0);  
          PMfindsbest_alhon[i,j] = 1;
      endif;

      if (lnodds_bayes > 0) and (thdens_m1 > thdens_m2);
          PMfindsbestth_alhon[i,j] = 1;
      elseif (lnodds_bayes < 0) and (thdens_m1 < thdens_m2);  
          PMfindsbestth_alhon[i,j] = 1;
      endif;
      
      j = j+1;
   endo;

   if fmod(i,nsim/nblocks)==0;
      tfin = date;
      "Elapsed Time to complete block"; etstr(ethsec(tstart,tfin));
      pause(1e-5);
      "";
      tstart = date;
   endif;
   
   i = i+1;
   
endo;

// Create Summary Statistics

M1true = meanc(Mtrue_vec .== 1);
M2true = meanc(Mtrue_vec .== 2);

// M1 is preferred

M1preferred_m1true = meanc(delif(M1preferred, Mtrue_vec .== 2))';
M1preferred_m2true = meanc(delif(M1preferred, Mtrue_vec .== 1))';

// Modelers are honest

M1reportstruth_m1true = meanc(delif(M1reportstruth, Mtrue_vec .== 2))';
M1reportstruth_m2true = meanc(delif(M1reportstruth, Mtrue_vec .== 1))';

M2reportstruth_m1true = meanc(delif(M2reportstruth, Mtrue_vec .== 2))';
M2reportstruth_m2true = meanc(delif(M2reportstruth, Mtrue_vec .== 1))';

// PM finds best model

PMfindsbest_m1true = meanc(delif(PMfindsbest, Mtrue_vec .== 2))';
PMfindsbest_m2true = meanc(delif(PMfindsbest, Mtrue_vec .== 1))';

PMfindsbestth_m1true = meanc(delif(PMfindsbestth, Mtrue_vec .== 2))';
PMfindsbestth_m2true = meanc(delif(PMfindsbestth, Mtrue_vec .== 1))';

PMfindsbestmining_m1true = meanc(delif(PMfindsbestmining, Mtrue_vec .== 2))';
PMfindsbestmining_m2true = meanc(delif(PMfindsbestmining, Mtrue_vec .== 1))';

// PM finds best model assuming modelers are honest

PMfindsbest_alhon_m1true = meanc(delif(PMfindsbest_alhon, Mtrue_vec .== 2))';
PMfindsbest_alhon_m2true = meanc(delif(PMfindsbest_alhon, Mtrue_vec .== 1))';

PMfindsbestth_alhon_m1true = meanc(delif(PMfindsbestth_alhon, Mtrue_vec .== 2))';
PMfindsbestth_alhon_m2true = meanc(delif(PMfindsbestth_alhon, Mtrue_vec .== 1))';

// Loss conditional on finding best model

PMprecision_m1true = ( sumc(delif(PMprecision, Mtrue_vec .== 2)) ./ sumc(delif(PMfindsbest, Mtrue_vec .== 2)) )'; 
PMprecision_m2true = ( sumc(delif(PMprecision, Mtrue_vec .== 1)) ./ sumc(delif(PMfindsbest, Mtrue_vec .== 1)) )'; 

PMprecisionth_m1true = ( sumc(delif(PMprecisionth, Mtrue_vec .== 2)) ./ sumc(delif(PMfindsbestth, Mtrue_vec .== 2)) )'; 
PMprecisionth_m2true = ( sumc(delif(PMprecisionth, Mtrue_vec .== 1)) ./ sumc(delif(PMfindsbestth, Mtrue_vec .== 1)) )'; 

PMprecisionmining_m1true = ( sumc(delif(PMprecisionmining, Mtrue_vec .== 2)) ./ sumc(delif(PMfindsbestmining, Mtrue_vec .== 2)) )';
PMprecisionmining_m2true = ( sumc(delif(PMprecisionmining, Mtrue_vec .== 1)) ./ sumc(delif(PMfindsbestmining, Mtrue_vec .== 1)) )';

// Total loss

PMtotalloss_m1true = meanc(delif(PMtotalloss, Mtrue_vec .== 2))';
PMtotalloss_m2true = meanc(delif(PMtotalloss, Mtrue_vec .== 1))';

PMtotallossth_m1true = meanc(delif(PMtotallossth, Mtrue_vec .== 2))';
PMtotallossth_m2true = meanc(delif(PMtotallossth, Mtrue_vec .== 1))';

PMtotallossmining_m1true = meanc(delif(PMtotallossmining, Mtrue_vec .== 2))';
PMtotallossmining_m2true = meanc(delif(PMtotallossmining, Mtrue_vec .== 1))';

// Screen Output

"M1 is true";
"----------";
"M1 is preferred   " M1preferred_m1true;
"M1 reports truth  " M1reportstruth_m1true;
"M2 reports truth  " M2reportstruth_m1true;
"";
"PM finds best     " PMfindsbest_m1true;
"PM finds best th  " PMfindsbestth_m1true;
"PM finds best mine" PMfindsbestmining_m1true;
"";
"PM precision      " PMprecision_m1true;
"PM precision th   " PMprecisionth_m1true;
"PM precision mine " PMprecisionmining_m1true;
"";
"PM finds best always honest      " PMfindsbest_alhon_m1true;
"PM finds best th always honest   " PMfindsbestth_alhon_m1true;
"";
"PM totalloss      " PMtotalloss_m1true;
"PM totalloss th   " PMtotallossth_m1true;
"PM totalloss mine " PMtotallossmining_m1true;
"";
"";

"M2 is true";
"----------";
"M1 is preferred   " M1preferred_m2true;
"M1 reports truth  " M1reportstruth_m2true;
"M2 reports truth  " M2reportstruth_m2true;
"";
"PM finds best     " PMfindsbest_m2true;
"PM finds best th  " PMfindsbestth_m2true;
"PM finds best mine" PMfindsbestmining_m2true;
"";
"PM precision      " PMprecision_m2true;
"PM precision th   " PMprecisionth_m2true;
"PM precision mine " PMprecisionmining_m2true;
"";
"PM finds best always honest      " PMfindsbest_alhon_m2true;
"PM finds best th always honest   " PMfindsbestth_alhon_m2true;
"";
"PM totalloss      " PMtotalloss_m2true;
"PM totalloss th   " PMtotallossth_m2true;
"PM totalloss mine " PMtotallossmining_m2true;

pause(1);

"Error Counts and conditional losses";
i = 1;
nu_vec = 0.1 | 1.1 | 2.1 | 3.1;
do until i > n_grid;
   "tau " tau_grid[i];
   "r   " r_grid[i];
   condloss   = zeros(1,4);
   condlossth = zeros(1,4);
   j = 1;
   do until j>4;
      condloss[1,j]   = meanc( delif(PMtotalloss[.,i], (selecterror[.,i] .ne (j-1))) );
      condlossth[1,j] = meanc( delif(PMtotallossth[.,i], (selecterrorth[.,i] .ne (j-1))) );
      j=j+1;
   endo;
   "Fraction of Errors   " counts(selecterror[.,i],nu_vec)'/nsim;
   "Fraction of Errors Th" counts(selecterrorth[.,i],nu_vec)'/nsim;
   "Conditional Loss     " condloss;
   "Conditional Loss   Th" condlossth;
   "";


   i = i+1;
endo;

pause(1);



// Store results on hard drive

Results_condM1 = r_grid' |
                 tau_grid' |
                 M1preferred_m1true |
                 M1reportstruth_m1true |
                 M2reportstruth_m1true |
                 PMfindsbest_m1true |
                 PMfindsbestth_m1true |
                 PMfindsbestmining_m1true |
                 PMprecision_m1true |
                 PMprecisionth_m1true |
                 PMprecisionmining_m1true |
                 PMfindsbest_alhon_m1true |
                 PMfindsbestth_alhon_m1true |
                 PMtotalloss_m1true |
                 PMtotallossth_m1true |
                 PMtotallossmining_m1true;                                  
                 
Results_condM2 = r_grid' |
                 tau_grid' |
                 M1preferred_m2true |
                 M1reportstruth_m2true |
                 M2reportstruth_m2true |
                 PMfindsbest_m2true |
                 PMfindsbestth_m2true |
                 PMfindsbestmining_m2true |
                 PMprecision_m2true |
                 PMprecisionth_m2true |
                 PMprecisionmining_m2true |
                 PMfindsbest_alhon_m2true |
                 PMfindsbestth_alhon_m2true |
                 PMtotalloss_m2true |
                 PMtotallossth_m2true |
                 PMtotallossmining_m2true                 
                 ;
output off;

opath    = resupath $+ "\\" $+ simrunstr;
chk_dir(opath);

oresults         = opath $+ "\\" $+ "ResultscondM1"; 
create fhresults = ^oresults with RESULTS, cols(Results_condM1), 8;
writer(fhresults,Results_condM1);
closeall;

oresults         = opath $+ "\\" $+ "ResultscondM2"; 
create fhresults = ^oresults with RESULTS, cols(Results_condM2), 8;
writer(fhresults,Results_condM2);
closeall;

end;
