/* ex1_procs.g
**  
** Procedures
**
*/

declare matrix nsim_thdens_;
declare matrix thfix_;
declare matrix thfixvalue_;

proc (9) = generatedata(pi01,siguu,lam1,lam2,n,sigmaxx,tsize);
// generate a draw from the marginal distribution of the data
local YYC,YYT,XXC,XXT,SC,ST,theta,beta,Mtrue;

SC = zeros(n/2,1);
ST = ones(n/2,1)*tsize;
XXC = rndn(n/2,2)*chol(sigmaxx);
XXT = rndn(n/2,2)*chol(sigmaxx);

if rndu(1,1) < pi01;
   /* select M1 */
   if thfix_ == 0;
      theta = (1/(sqrt(n)*lam1))*rndn(1,1);
   else;
      theta = thfixvalue_;
   endif;
   YYC = theta*(XXC[.,1]+SC) + sqrt(siguu)*rndn(n/2,1);
   YYT = theta*(XXT[.,1]+ST) + sqrt(siguu)*rndn(n/2,1);
   Mtrue = 1;
   beta = theta ~ 0;
else;
   /* select M2 */
   if thfix_ == 0;
      theta = (1/(sqrt(n)*lam2))*rndn(1,1);
   else;
      theta = thfixvalue_;
   endif;
   YYC = theta*(XXC[.,2]+SC) + sqrt(siguu)*rndn(n/2,1);
   YYT = theta*(XXT[.,2]+ST) + sqrt(siguu)*rndn(n/2,1);
   Mtrue = 2;
   beta = 0 ~ theta;
endif;

retp(YYC,YYT,XXC,XXT,SC,ST,theta,beta,Mtrue);
endp;


/**********************************************************/

proc (9) = bayesiananalysis(YYC,YYT,XXC,XXT,SC,ST,p01,lam1,lam2);
// we currently assume that siguu = 1 
local YYtilde, XX1tilde, XX2tilde, XX1, XX2,
      theta_bar1, theta_bar2, V_bar1, V_bar2,
      theta_bar10, theta_bar20,
      lnpYY1, lnpYY2, odds, nobs, delta_fit, delta_penalty;

YYtilde    = YYC | YYT;
YYtilde    = YYtilde - meanc(YYtilde)';
nobs       = rows(YYtilde);

// Posterior of model M1
XX1tilde   = (XXC[.,1] + SC)|(XXT[.,1] + ST);
XX1tilde   = XX1tilde - meanc(XX1tilde)';
theta_bar1 = (XX1tilde'*YYtilde)/(XX1tilde'*XX1tilde + nobs*lam1^2);
V_bar1     = inv(XX1tilde'*XX1tilde+nobs*lam1^2);
lnpYY1     = -0.5*nobs*ln(2*pi) + 0.5*ln(nobs*lam1^2) - 0.5*ln(det(XX1tilde'*XX1tilde+nobs*lam1^2))
             -0.5*(YYtilde'*YYtilde - YYtilde'*XX1tilde*theta_bar1);
             
// Posterior of model M10
XX1   = (XXC[.,1] ~ SC)|(XXT[.,1] ~ ST);
XX1   = XX1 - meanc(XX1)';
theta_bar10 = (XX1'*YYtilde)/(XX1'*XX1 + eye(2)*nobs*lam1^2);

// Posterior of model M2
XX2tilde   = (XXC[.,2] + SC)|(XXT[.,2] + ST);
XX2tilde   = XX2tilde - meanc(XX2tilde)';
theta_bar2 = (XX2tilde'*YYtilde)/(XX2tilde'*XX2tilde + nobs*lam2^2);
V_bar2     = inv(XX2tilde'*XX2tilde+nobs*lam2^2);
lnpYY2     = -0.5*nobs*ln(2*pi) + 0.5*ln(nobs*lam2^2) - 0.5*ln(det(XX2tilde'*XX2tilde+nobs*lam2^2))
             -0.5*(YYtilde'*YYtilde - YYtilde'*XX2tilde*theta_bar2);

// Posterior of model M20
XX2   = (XXC[.,2] ~ SC)|(XXT[.,2] ~ ST);
XX2   = XX2 - meanc(XX2)';
theta_bar20 = (XX2'*YYtilde)/(XX2'*XX2 + eye(2)*nobs*lam2^2);


odds       = lnpYY1-lnpYY2;

delta_fit     =  -0.5*(YYtilde'*YYtilde - YYtilde'*XX1tilde*theta_bar1)
                 +0.5*(YYtilde'*YYtilde - YYtilde'*XX2tilde*theta_bar2);
delta_penalty =   0.5*ln(nobs*lam1^2) - 0.5*ln(det(XX1tilde'*XX1tilde+nobs*lam1^2))
                 -0.5*ln(nobs*lam2^2) + 0.5*ln(det(XX2tilde'*XX2tilde+nobs*lam2^2));

retp(theta_bar1,V_bar1,theta_bar2,V_bar2,theta_bar10,theta_bar20,odds,delta_fit,delta_penalty);
endp;

/**********************************************************/

proc (16) = samplesplitting(YYC,YYT,XXC,XXT,SC,ST,lam1,lam2,r,tau);
// we currently assume that siguu = 1 
local YYtilde, Y_r, S_r, S_p,
      YYtilde_r, XX1tilde_r, XX2tilde_r, YY_r, XX1_r, XX2_r,
      YYtilde_p, XX1tilde_p, XX2tilde_p, YY_p, XX1_p, XX2_p,
      theta_tilde1_r, theta_tilde2_r, V_tilde1_r, V_tilde2_r,
      theta_bar1_r, theta_bar2_r, V_bar1_r, V_bar2_r,
      lnpYY_tilde1_r, lnpYY_tilde2_r, lnpYY_bar1_r, lnpYY_bar2_r,
      mth_m1tilde, sth_m1tilde, 
      mth_m2tilde, sth_m2tilde, 
      mth_m1bar, sth_m1bar, 
      mth_m2bar, sth_m2bar, 
      nobs, nr, nrC, nrT, np, npC, npT,
      S_all, Y_all;

/* split sample
*/ 
YYtilde    = YYC | YYT;
YYtilde    = YYtilde - meanc(YYtilde)';
nobs       = rows(YYtilde);


// Split Sample 
nr  = ceil(nobs*r); // estimation sample
np  = nobs-nr;      // hold-out sample 
nrT = ceil(nr*tau);
nrC = nr-nrT;
npT = ceil(np*tau);
npC = np-npT;

// adjustments
if nrC > nobs/2;
   nrC = nobs/2;
   nrT = nr-nrC;
   npT = nobs/2-nrT;
   npC = 0;
elseif nrT > nobs/2;
   nrT = nobs/2;
   nrC = nr-nrT;
   npC = nobs/2-nrC;
   npT = 0;
endif;

if nrC == 0;
   YYtilde_r  = YYT[1:nrT,.];
   XX1tilde_r = XXT[1:nrT,1] + ST[1:nrT,.];
   XX1_r      = XXT[1:nrT,1] ~ ST[1:nrT,.];
   XX2tilde_r = XXT[1:nrT,2] + ST[1:nrT,.];
   XX2_r      = XXT[1:nrT,2] ~ ST[1:nrT,.];
   if nrT < nobs/2;
      YYtilde_p  = YYT[nrT+1:nobs/2,.] | YYC ;
      XX1tilde_p = (XXT[nrT+1:nobs/2,1] + ST[nrT+1:nobs/2,.]) | (XXC[.,1] + SC[.,1]);
      XX1_p      = (XXT[nrT+1:nobs/2,1] ~ ST[nrT+1:nobs/2,.]) | (XXC[.,1] ~ SC[.,1]);
      XX2tilde_p = (XXT[nrT+1:nobs/2,2] + ST[nrT+1:nobs/2,.]) | (XXC[.,2] + SC[.,1]);
      XX2_p      = (XXT[nrT+1:nobs/2,2] ~ ST[nrT+1:nobs/2,.]) | (XXC[.,2]  ~SC[.,1]);
   else;
      YYtilde_p  =  YYC ;
      XX1tilde_p = (XXC[.,1] + SC[.,1]);
      XX1_p      = (XXC[.,1] ~ SC[.,1]);
      XX2tilde_p = (XXC[.,2] + SC[.,1]);
      XX2_p      = (XXC[.,2]  ~SC[.,1]);
   endif;   
   
elseif nrT == 0;
   YYtilde_r  = YYC[1:nrC,.];
   XX1tilde_r = XXC[1:nrC,1] + SC[1:nrC,.];
   XX1_r      = XXC[1:nrC,1] ~ SC[1:nrC,.];
   XX2tilde_r = XXC[1:nrC,2] + SC[1:nrC,.];
   XX2_r      = XXC[1:nrC,2] ~ SC[1:nrC,.];
   if nrC < nobs/2;
      YYtilde_p  = YYC[nrC+1:nobs/2,.] | YYT ;
      XX1tilde_p = (XXC[nrC+1:nobs/2,1] + SC[nrC+1:nobs/2,.]) | (XXT[.,1] + ST[.,1]);
      XX1_p      = (XXC[nrC+1:nobs/2,1] ~ SC[nrC+1:nobs/2,.]) | (XXT[.,1] ~ ST[.,1]);
      XX2tilde_p = (XXC[nrC+1:nobs/2,2] + SC[nrC+1:nobs/2,.]) | (XXT[.,2] + ST[.,1]);
      XX2_p      = (XXC[nrC+1:nobs/2,2] ~ SC[nrC+1:nobs/2,.]) | (XXT[.,2] ~ ST[.,1]);
   else;
      YYtilde_p  =  YYT ;
      XX1tilde_p = (XXT[.,1] + ST[.,1]);
      XX1_p      = (XXT[.,1] ~ ST[.,1]);
      XX2tilde_p = (XXT[.,2] + ST[.,1]);
      XX2_p      = (XXT[.,2] ~ ST[.,1]);
   endif;   
           
else;
   YYtilde_r = YYC[1:nrC,.] | YYT[1:nrT,.];
   XX1tilde_r= (XXC[1:nrC,1] + SC[1:nrC,.])|(XXT[1:nrT,1] + ST[1:nrT,.]);
   XX1_r     = (XXC[1:nrC,1] ~ SC[1:nrC,.])|(XXT[1:nrT,1] ~ ST[1:nrT,.]);
   XX2tilde_r= (XXC[1:nrC,2] + SC[1:nrC,.])|(XXT[1:nrT,2] + ST[1:nrT,.]);
   XX2_r     = (XXC[1:nrC,2] ~ SC[1:nrC,.])|(XXT[1:nrT,2] ~ ST[1:nrT,.]);
   if nrC == nobs/2;
      YYtilde_p  =  YYT[nrT+1:nobs/2,.] ;
      XX1tilde_p = (XXT[nrT+1:nobs/2,1] + ST[nrT+1:nobs/2,1]);
      XX1_p      = (XXT[nrT+1:nobs/2,1] ~ ST[nrT+1:nobs/2,1]);
      XX2tilde_p = (XXT[nrT+1:nobs/2,2] + ST[nrT+1:nobs/2,1]);
      XX2_p      = (XXT[nrT+1:nobs/2,2] ~ ST[nrT+1:nobs/2,1]);
   elseif nrT == nobs/2;
      YYtilde_p  =  YYC[nrC+1:nobs/2,.] ;
      XX1tilde_p = (XXC[nrC+1:nobs/2,1] + SC[nrC+1:nobs/2,1]);
      XX1_p      = (XXC[nrC+1:nobs/2,1] ~ SC[nrC+1:nobs/2,1]);
      XX2tilde_p = (XXC[nrC+1:nobs/2,2] + SC[nrC+1:nobs/2,1]);
      XX2_p      = (XXC[nrC+1:nobs/2,2] ~ SC[nrC+1:nobs/2,1]);
   else;
      YYtilde_p  = YYC[nrC+1:nobs/2,.] | YYT[nrT+1:nobs/2,.] ;
      XX1tilde_p = (XXC[nrC+1:nobs/2,1] + SC[nrC+1:nobs/2,.]) | (XXT[nrT+1:nobs/2,1] + ST[nrT+1:nobs/2,1]);
      XX1_p      = (XXC[nrC+1:nobs/2,1] ~ SC[nrC+1:nobs/2,.]) | (XXT[nrT+1:nobs/2,1] ~ ST[nrT+1:nobs/2,1]);
      XX2tilde_p = (XXC[nrC+1:nobs/2,2] + SC[nrC+1:nobs/2,.]) | (XXT[nrT+1:nobs/2,2] + ST[nrT+1:nobs/2,1]);
      XX2_p      = (XXC[nrC+1:nobs/2,2] ~ SC[nrC+1:nobs/2,.]) | (XXT[nrT+1:nobs/2,2] ~ ST[nrT+1:nobs/2,1]);
   endif;         
   
endif;


Y_r          = YYtilde_r;
S_r          = XX1_r[.,cols(XX1_r)];
S_p          = XX1_p[.,cols(XX1_p)];

S_all = S_r | S_p;
S_all = S_all - meanc(S_all);
Y_all = YYtilde_r | YYtilde_p;
Y_all = Y_all - meanc(Y_all)';
//nrC nrT (nrC+nrT) inv(S_all'*S_all)*S_all'*Y_all;


YYtilde_r    = YYtilde_r  - meanc(YYtilde_r)';
XX1tilde_r   = XX1tilde_r - meanc(XX1tilde_r)';
XX2tilde_r   = XX2tilde_r - meanc(XX2tilde_r)';
YY_r         = YYtilde_r;
XX1_r        = XX1_r - meanc(XX1_r)';
XX2_r        = XX2_r - meanc(XX2_r)';
YY_p         = YYtilde_p;



// Posterior of model M1
theta_tilde1_r = (XX1tilde_r'*YYtilde_r)/(XX1tilde_r'*XX1tilde_r + nobs*lam1^2);
V_tilde1_r     = inv(XX1tilde_r'*XX1tilde_r+nobs*lam1^2);
lnpYY_tilde1_r = -0.5*rows(YYtilde_r)*ln(2*pi) + 0.5*ln(nobs*lam1^2) 
                 -0.5*ln(det(XX1tilde_r'*XX1tilde_r+nobs*lam1^2))
                 -0.5*(YYtilde_r'*YYtilde_r - YYtilde_r'*XX1tilde_r*theta_tilde1_r);

{mth_m1tilde,sth_m1tilde} 
              = thetadens(Y_r,S_r,XX1tilde_p,S_p,
                theta_tilde1_r,V_tilde1_r);

// Posterior of model M1 reference
theta_bar1_r   = (XX1_r'*YY_r)/(XX1_r'*XX1_r + nobs*lam1^2*eye(cols(XX1_r)));
V_bar1_r       = inv(XX1_r'*XX1_r+nobs*lam1^2*eye(cols(XX1_r)));
lnpYY_bar1_r   = -0.5*rows(YY_r)*ln(2*pi) + 0.5*ln(det(nobs*lam1^2*eye(cols(XX1_r)))) 
                 -0.5*ln(det(XX1_r'*XX1_r+nobs*lam1^2*eye(cols(XX1_r))))
                 -0.5*(YY_r'*YY_r - YY_r'*XX1_r*theta_bar1_r);

{mth_m1bar,sth_m1bar} 
              = thetadens(Y_r,S_r,XX1_p,S_p,theta_bar1_r,V_bar1_r);


// Posterior of model M2
theta_tilde2_r = (XX2tilde_r'*YYtilde_r)/(XX2tilde_r'*XX2tilde_r + nobs*lam2^2);
V_tilde2_r     = inv(XX2tilde_r'*XX2tilde_r+nobs*lam2^2);
lnpYY_tilde2_r = -0.5*rows(YYtilde_r)*ln(2*pi) + 0.5*ln(nobs*lam2^2) - 0.5*ln(det(XX2tilde_r'*XX2tilde_r+nobs*lam2^2))
                 -0.5*(YYtilde_r'*YYtilde_r - YYtilde_r'*XX2tilde_r*theta_tilde2_r);

{mth_m2tilde,sth_m2tilde} 
              = thetadens(Y_r,S_r,XX2tilde_p,S_p,theta_tilde2_r,V_tilde2_r);


// Posterior of model M2 reference
theta_bar2_r   = (XX2_r'*YY_r)/(XX2_r'*XX2_r + nobs*lam2^2*eye(cols(XX2_r)));
V_bar2_r       = inv(XX2_r'*XX2_r+nobs*lam2^2*eye(cols(XX2_r)));
lnpYY_bar2_r   = -0.5*rows(YY_r)*ln(2*pi) + 0.5*ln(det(nobs*lam2^2*eye(cols(XX2_r)))) 
                 -0.5*ln(det(XX2_r'*XX2_r+nobs*lam2^2*eye(cols(XX2_r))))
                 -0.5*(YY_r'*YY_r - YY_r'*XX2_r*theta_bar2_r);

{mth_m2bar,sth_m2bar} 
              = thetadens(Y_r,S_r,XX2_p,S_p,theta_bar2_r,V_bar2_r);

retp(lnpYY_tilde1_r,lnpYY_bar1_r,lnpYY_tilde2_r,lnpYY_bar2_r,
     theta_tilde1_r,theta_tilde2_r,theta_bar1_r,theta_bar2_r,
     mth_m1tilde,sth_m1tilde,mth_m1bar,sth_m1bar,
     mth_m2tilde,sth_m2tilde,mth_m2bar,sth_m2bar);
endp;

/**********************************************************/


proc (7) = datamining(YYC,YYT,XXC,XXT,SC,ST,p01);
// we currently assume that siguu = 1 
local YYtilde, XX1tilde, XX2tilde,
      theta_bar1, theta_bar2, V_bar1, V_bar2,
      lnpYY1, lnpYY2, odds, nobs, delta_fit, delta_penalty;

YYtilde    = YYC | YYT;
YYtilde    = YYtilde - meanc(YYtilde)';
nobs       = rows(YYtilde);

// Posterior of model M1
XX1tilde   = (XXC[.,1] ~ SC)|(XXT[.,1] ~ ST);
XX1tilde   = XX1tilde - meanc(XX1tilde)';
theta_bar1 = (XX1tilde'*YYtilde)/(XX1tilde'*XX1tilde);
V_bar1     = inv(2*XX1tilde'*XX1tilde);
lnpYY1     = -0.5*nobs*ln(2*pi) -0.5*ln(2)
             -0.5*(YYtilde'*YYtilde - YYtilde'*XX1tilde*theta_bar1);

// Posterior of model M2
XX2tilde   = (XXC[.,2] ~ SC)|(XXT[.,2] ~ ST);
XX2tilde   = XX2tilde - meanc(XX2tilde)';
theta_bar2 = (XX2tilde'*YYtilde)/(XX2tilde'*XX2tilde);
V_bar2     = inv(2*XX2tilde'*XX2tilde);
lnpYY2     = -0.5*nobs*ln(2*pi) -0.5*ln(2) 
             -0.5*(YYtilde'*YYtilde - YYtilde'*XX2tilde*theta_bar2);

delta_fit  = -0.5*(YYtilde'*YYtilde - YYtilde'*XX1tilde*theta_bar1)
             +0.5*(YYtilde'*YYtilde - YYtilde'*XX2tilde*theta_bar2);

delta_penalty = 0;

odds        = lnpYY1-lnpYY2;

retp(theta_bar1,V_bar1,theta_bar2,V_bar2,odds,delta_fit,delta_penalty);
endp;

/**********************************************************/


proc (2) = thetadens(YY_r,S_r,XX_p,S_p,beta_bar,V_bar);
local nsim,theta_sim,i,beta,YY_p,S,YY;

nsim = nsim_thdens_;
theta_sim = zeros(nsim,1);

i = 1;
do until i > nsim;
   beta = beta_bar + cholpsd(V_bar) * rndn(rows(beta_bar),1);
   YY_p = XX_p*beta+rndn(rows(XX_p),1);
   S  = S_r | S_p;
   YY = YY_r | YY_p;
   S  = S - meanc(S)';
   YY = YY - meanc(YY)';
   theta_sim[i] = inv(S'*S)*S'YY;
   i = i+1;
endo;

retp(meanc(theta_sim),stdc(theta_sim));
endp;

